// Solutions to the Advent Of Code 2024.
// Copyright (C) 2024  Stefan Mller
//
// This program is free software: you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along with
// this program.  If not, see <http://www.gnu.org/licenses/>.

#include "pch.h"
#include "CppUnitTest.h"

#include "../Demo.h"
#include "../Solver.h"
#include "../SolverEngine.h"

using namespace Microsoft::VisualStudio::CppUnitTestFramework;

namespace UnitTests
{
    TEST_CLASS(DemoTests)
    {
    public:
        TEST_METHOD(FullData1)
        {
            SolverEngine solverEngine{ getInputPaths() };
            std::unique_ptr<Solver> solver{ std::make_unique<Demo>() };
            solverEngine.run(*solver);
            assertAreEqual(66, solver->getResultPart1());
        }

        TEST_METHOD(FullData2)
        {
            SolverEngine solverEngine{ getInputPaths() };
            std::unique_ptr<Solver> solver = std::make_unique<Demo>();
            solverEngine.run(*solver);
            assertAreEqual(39916800, solver->getResultPart2());
        }

        TEST_METHOD(ExampleData1)
        {
            SolverEngine solverEngine{ getExampleInputPaths() };
            std::unique_ptr<Solver> solver = std::make_unique<Demo>();
            solverEngine.run(*solver);
            assertAreEqual(10, solver->getResultPart1());
        }

        TEST_METHOD(ExampleData2)
        {
            SolverEngine solverEngine{ getExampleInputPaths() };
            std::unique_ptr<Solver> solver = std::make_unique<Demo>();
            solverEngine.run(*solver);
            assertAreEqual(24, solver->getResultPart2());
        }

    private:
        std::vector<std::string> getInputPaths() const
        {
            return std::vector<std::string>{ "../../data", "../../../../data" };
        }

        std::vector<std::string> getExampleInputPaths() const
        {
            return std::vector<std::string>{ "../../data/examples", "../../../../data/examples" };
        }

        void assertAreEqual(const long long int expected, const long long int actual) const
        {
            std::wstring message = L"Expected: <" + std::to_wstring(expected) + L"> Actual: <" + std::to_wstring(actual) + L">";
            Assert::IsTrue(expected == actual, message.c_str());
        }
    };
}
